@@ -19,19 +19,19 @@ import os
 import re
 import shutil
 import subprocess
-from .._pathlib import Path
+from pathlib import Path
 
 from ..mesonlib import OrderedSet, join_args
-from .base import (
-    DependencyException, DependencyMethods, ConfigToolDependency,
-    PkgConfigDependency, factory_methods
-)
+from .base import DependencyException, DependencyMethods
+from .configtool import ConfigToolDependency
+from .pkgconfig import PkgConfigDependency
+from .factory import factory_methods
 import typing as T
 
 if T.TYPE_CHECKING:
-    from .base import Dependency
-    from ..envconfig import MachineChoice
+    from .factory import DependencyGenerator
     from ..environment import Environment
+    from ..mesonlib import MachineChoice
 
 
 class HDF5PkgConfigDependency(PkgConfigDependency):
@@ -41,7 +41,7 @@ class HDF5PkgConfigDependency(PkgConfigDependency):
     def __init__(self, name: str, environment: 'Environment', kwargs: T.Dict[str, T.Any], language: T.Optional[str] = None) -> None:
         language = language or 'c'
         if language not in {'c', 'cpp', 'fortran'}:
-            raise DependencyException('Language {} is not supported with HDF5.'.format(language))
+            raise DependencyException(f'Language {language} is not supported with HDF5.')
 
         super().__init__(name, environment, kwargs, language)
         if not self.is_found:
@@ -92,17 +92,17 @@ class HDF5ConfigToolDependency(ConfigToolDependency):
     def __init__(self, name: str, environment: 'Environment', kwargs: T.Dict[str, T.Any], language: T.Optional[str] = None) -> None:
         language = language or 'c'
         if language not in {'c', 'cpp', 'fortran'}:
-            raise DependencyException('Language {} is not supported with HDF5.'.format(language))
+            raise DependencyException(f'Language {language} is not supported with HDF5.')
 
         if language == 'c':
             cenv = 'CC'
-            tools = ['h5cc']
+            tools = ['h5cc', 'h5pcc']
         elif language == 'cpp':
             cenv = 'CXX'
-            tools = ['h5c++']
+            tools = ['h5c++', 'h5pc++']
         elif language == 'fortran':
             cenv = 'FC'
-            tools = ['h5fc']
+            tools = ['h5fc', 'h5pfc']
         else:
             raise DependencyException('How did you get here?')
 
@@ -117,17 +117,20 @@ class HDF5ConfigToolDependency(ConfigToolDependency):
         # linkers.
         compiler = environment.coredata.compilers[for_machine][language]
         try:
-            os.environ['HDF5_{}'.format(cenv)] = join_args(compiler.get_exelist())
-            os.environ['HDF5_{}LINKER'.format(cenv)] = join_args(compiler.get_linker_exelist())
+            os.environ[f'HDF5_{cenv}'] = join_args(compiler.get_exelist())
+            os.environ[f'HDF5_{cenv}LINKER'] = join_args(compiler.get_linker_exelist())
             super().__init__(name, environment, nkwargs, language)
         finally:
-            del os.environ['HDF5_{}'.format(cenv)]
-            del os.environ['HDF5_{}LINKER'.format(cenv)]
+            del os.environ[f'HDF5_{cenv}']
+            del os.environ[f'HDF5_{cenv}LINKER']
         if not self.is_found:
             return
 
-        args = self.get_config_value(['-show', '-noshlib' if kwargs.get('static', False) else '-shlib'], 'args')
-        for arg in args[1:]:
+        # We first need to call the tool with -c to get the compile arguments
+        # and then without -c to get the link arguments.
+        args = self.get_config_value(['-show', '-c'], 'args')[1:]
+        args += self.get_config_value(['-show', '-noshlib' if kwargs.get('static', False) else '-shlib'], 'args')[1:]
+        for arg in args:
             if arg.startswith(('-I', '-f', '-D')) or arg == '-pthread':
                 self.compile_args.append(arg)
             elif arg.startswith(('-L', '-l', '-Wl')):
@@ -140,7 +143,7 @@ class HDF5ConfigToolDependency(ConfigToolDependency):
             nkwargs = kwargs.copy()
             nkwargs['language'] = 'c'
             # I'm being too clever for mypy and pylint
-            self.is_found = self._add_sub_dependency(hdf5_factory(environment, for_machine, nkwargs))  # type: ignore  # pylint: disable=no-value-for-parameter
+            self.is_found = self._add_sub_dependency(hdf5_factory(environment, for_machine, nkwargs))  # pylint: disable=no-value-for-parameter
 
     def _sanitize_version(self, ver: str) -> str:
         v = re.search(r'\s*HDF5 Version: (\d+\.\d+\.\d+)', ver)
@@ -149,9 +152,9 @@ class HDF5ConfigToolDependency(ConfigToolDependency):
 
 @factory_methods({DependencyMethods.PKGCONFIG, DependencyMethods.CONFIG_TOOL})
 def hdf5_factory(env: 'Environment', for_machine: 'MachineChoice',
-                 kwargs: T.Dict[str, T.Any], methods: T.List[DependencyMethods]) -> T.List[T.Callable[[], 'Dependency']]:
+                 kwargs: T.Dict[str, T.Any], methods: T.List[DependencyMethods]) -> T.List['DependencyGenerator']:
     language = kwargs.get('language')
-    candidates = []  # type: T.List[T.Callable[[], Dependency]]
+    candidates: T.List['DependencyGenerator'] = []
 
     if DependencyMethods.PKGCONFIG in methods:
         # Use an ordered set so that these remain the first tried pkg-config files
@@ -164,7 +167,7 @@ def hdf5_factory(env: 'Environment', for_machine: 'MachineChoice',
                                     universal_newlines=True)
             if ret.returncode == 0:
                 for pkg in ret.stdout.split('\n'):
-                    if pkg.startswith(('hdf5')):
+                    if pkg.startswith('hdf5'):
                         pkgconfig_files.add(pkg.split(' ', 1)[0])
 
         for pkg in pkgconfig_files:
