@@ -161,7 +161,8 @@ class PythonSystemDependency(SystemDependency, _PythonDependencyBase):
                     libpath = Path(f'python{vernum}.dll')
                 else:
                     libpath = Path('libs') / f'python{vernum}.lib'
-            lib = Path(self.variables.get('base')) / libpath
+            # base_prefix to allow for virtualenvs.
+            lib = Path(self.variables.get('base_prefix')) / libpath
         elif self.platform == 'mingw':
             if self.static:
                 libname = self.variables.get('LIBRARY')
@@ -318,8 +319,11 @@ def links_against_libpython():
     cmd.ensure_finalized()
     return bool(cmd.get_libraries(Extension('dummy', [])))
 
+variables = sysconfig.get_config_vars()
+variables.update({'base_prefix': getattr(sys, 'base_prefix', sys.prefix)})
+
 print(json.dumps({
-  'variables': sysconfig.get_config_vars(),
+  'variables': variables,
   'paths': paths,
   'install_paths': install_paths,
   'sys_paths': sys.path,
@@ -338,7 +342,7 @@ if T.TYPE_CHECKING:
         link_libpython: bool
         paths: T.Dict[str, str]
         platform: str
-        suffix : str
+        suffix: str
         variables: T.Dict[str, str]
         version: str
 
@@ -399,16 +403,19 @@ class PythonExternalProgram(ExternalProgram):
             return False
 
     def _get_path(self, state: T.Optional['ModuleState'], key: str) -> None:
-        if state:
-            value = state.get_option(f'{key}dir', module='python')
-            if value:
-                return value
-        user_dir = str(Path.home())
-        sys_paths = self.info['sys_paths']
         rel_path = self.info['install_paths'][key][1:]
-        if not any(p.endswith(rel_path) for p in sys_paths if not p.startswith(user_dir)):
-            mlog.warning('Broken python installation detected. Python files',
-                         'installed by Meson might not be found by python interpreter.\n',
+        if not state:
+            # This happens only from run_project_tests.py
+            return rel_path
+        value = state.get_option(f'{key}dir', module='python')
+        if value:
+            return value
+        # Use python's path relative to prefix, and warn if that's not a location
+        # python will lookup for modules.
+        abs_path = Path(state.get_option('prefix'), rel_path)
+        sys_paths = [Path(i) for i in self.info['sys_paths']]
+        if abs_path not in sys_paths:
+            mlog.warning('Python files installed by Meson might not be found by python interpreter.\n',
                          f'This warning can be avoided by setting "python.{key}dir" option.',
                          once=True)
         return rel_path
@@ -494,18 +501,18 @@ class PythonInstallation(ExternalProgramHolder):
     @noPosargs
     def dependency_method(self, args: T.List['TYPE_var'], kwargs: 'TYPE_kwargs') -> 'Dependency':
         disabled, required, feature = extract_required_kwarg(kwargs, self.subproject)
-
         # it's theoretically (though not practically) possible for the else clse
         # to not bind dep, let's ensure it is.
-        dep: 'Dependency' = NotFoundDependency(self.interpreter.environment)
+        dep: 'Dependency' = NotFoundDependency('python', self.interpreter.environment)
         if disabled:
             mlog.log('Dependency', mlog.bold('python'), 'skipped: feature', mlog.bold(feature), 'disabled')
         else:
+            new_kwargs = kwargs.copy()
+            new_kwargs['required'] = False
+            methods = process_method_kw({DependencyMethods.PKGCONFIG, DependencyMethods.SYSTEM}, kwargs)
             for d in python_factory(self.interpreter.environment,
-                                      MachineChoice.BUILD if kwargs.get('native', False) else MachineChoice.HOST,
-                                      kwargs,
-                                      process_method_kw({DependencyMethods.PKGCONFIG, DependencyMethods.SYSTEM}, kwargs),
-                                      self):
+                                    MachineChoice.BUILD if kwargs.get('native', False) else MachineChoice.HOST,
+                                    new_kwargs, methods, self):
                 dep = d()
                 if dep.found():
                     break
